import { Clipboard, Dialog, showDialog } from '@jupyterlab/apputils';
// import { PathExt } from '@jupyterlab/coreutils';
// import { DocumentWidget } from '@jupyterlab/docregistry';
// import { FileEditor } from '@jupyterlab/fileeditor';
import { NotebookPanel } from '@jupyterlab/notebook';
import { LabIcon, pythonIcon, } from '@jupyterlab/ui-components';
import { CodeCellModel, MarkdownCell, CodeCell } from '@jupyterlab/cells';
import { Widget } from '@lumino/widgets';
// import { MimeData } from '@lumino/coreutils';
import React from 'react';
import { FilterTools } from './FilterTools';
// import { showPreview } from './PreviewSnippet';
import { showMoreOptions } from './MoreOptions';
import moreSVGstr from '../style/icon/jupyter_moreicon.svg';
import { requestAPI } from './jupyterlab-dbaccess';
/**
 * The CSS class added to code snippet widget.
 */
// const CODE_SNIPPETS_HEADER_CLASS = 'jp-codeSnippetsHeader';
// const CODE_SNIPPET_TITLE = 'jp-codeSnippet-title';
const CODE_SNIPPETS_CONTAINER = 'jp-codeSnippetsContainer';
const DISPLAY_NAME_CLASS = 'jp-codeSnippetsContainer-name';
const BUTTON_CLASS = 'jp-codeSnippetsContainer-button';
const TITLE_CLASS = 'jp-codeSnippetsContainer-title';
const ACTION_BUTTONS_WRAPPER_CLASS = 'jp-codeSnippetsContainer-action-buttons';
const ACTION_BUTTON_CLASS = 'jp-codeSnippetsContainer-actionButton';
const SEARCH_BOLD = 'jp-codeSnippet-search-bolding';
// const SNIPPET_DRAG_IMAGE = 'jp-codeSnippet-drag-image';
// const CODE_SNIPPET_DRAG_HOVER = 'jp-codeSnippet-drag-hover';
// const CODE_SNIPPET_DRAG_HOVER_SELECTED = 'jp-codeSnippet-drag-hover-selected';
const CODE_SNIPPET_METADATA = 'jp-codeSnippet-metadata';
// const CODE_SNIPPET_DESC = 'jp-codeSnippet-description';
// const CODE_SNIPPET_EDITOR = 'jp-codeSnippet-editor';
const CODE_SNIPPET_MORE_OPTIONS = 'jp-codeSnippet-options';
const CODE_SNIPPET_MORE_OTPIONS_CONTENT = 'jp-codeSnippet-more-options-content';
const CODE_SNIPPET_MORE_OTPIONS_COPY = 'jp-codeSnippet-more-options-copy';
const CODE_SNIPPET_MORE_OTPIONS_INSERT = 'jp-codeSnippet-more-options-insert';
// const CODE_SNIPPET_MORE_OTPIONS_EDIT = 'jp-codeSnippet-more-options-edit';
// const CODE_SNIPPET_MORE_OTPIONS_DELETE = 'jp-codeSnippet-more-options-delete';
// const CODE_SNIPPET_CREATE_NEW_BTN = 'jp-createSnippetBtn';
/**
 * The threshold in pixels to start a drag event.
 */
// const DRAG_THRESHOLD = 5;
/**
 * A class used to indicate a snippet item.
 */
const CODE_SNIPPET_ITEM = 'jp-codeSnippet-item';
/**
 * The mimetype used for Jupyter cell data.
 */
// const JUPYTER_CELL_MIME = 'application/vnd.jupyter.cells';
/**
 * Icon for more options
 */
const moreOptionsIcon = new LabIcon({
    name: 'custom-ui-components:moreOptions',
    svgstr: moreSVGstr
});
/**
 * A React Component for code-snippets display list.
 */
export class CodeSnippetDisplay extends React.Component {
    constructor(props) {
        super(props);
        // Handle code snippet insert into an editor
        this.insertCodeSnippet = async (snippet) => {
            var _a, _b, _c;
            const widget = this.props.getCurrentWidget();
            const snippetStr = `obs_utils.read_dataset(${snippet.code.join('\n')})`;
            console.log(snippetStr);
            if (widget instanceof NotebookPanel) {
                const notebookWidget = widget;
                const notebookCell = notebookWidget.content.activeCell;
                // editor
                const notebookCellEditor = notebookCell.editor;
                if (notebookCell instanceof CodeCell) {
                    const kernelInfo = await ((_b = (_a = notebookWidget.sessionContext.session) === null || _a === void 0 ? void 0 : _a.kernel) === null || _b === void 0 ? void 0 : _b.info);
                    const kernelLanguage = ((_c = kernelInfo) === null || _c === void 0 ? void 0 : _c.language_info.name) || '';
                    this.verifyLanguageAndInsert(snippet, kernelLanguage, notebookCellEditor);
                }
                else if (notebookCell instanceof MarkdownCell) {
                    // Wrap snippet into a code block when inserting it into a markdown cell
                    notebookCellEditor.replaceSelection('```' + snippet.language + '\n' + snippetStr + '\n```');
                }
                else {
                    notebookCellEditor.replaceSelection(snippetStr);
                }
            }
            else {
                this.showErrDialog('Code snippet insert failed: Unsupported widget');
            }
        };
        // Handle language compatibility between code snippet and editor
        this.verifyLanguageAndInsert = async (snippet, editorLanguage, editor) => {
            const snippetStr = `obs_utils.read_dataset(${snippet.code.join('\n')})`;
            if (editorLanguage &&
                snippet.language.toLowerCase() !== editorLanguage.toLowerCase()) {
                const result = await this.showWarnDialog(editorLanguage, snippet.name);
                if (result.button.accept) {
                    editor.replaceSelection(snippetStr);
                }
            }
            else {
                // Language match or editorLanguage is unavailable
                editor.replaceSelection(snippetStr);
            }
        };
        // Display warning dialog when inserting a code snippet incompatible with editor's language
        this.showWarnDialog = async (editorLanguage, snippetName) => {
            return showDialog({
                title: 'Warning',
                body: 'Code snippet "' +
                    snippetName +
                    '" is incompatible with ' +
                    editorLanguage +
                    '. Continue?',
                buttons: [Dialog.cancelButton(), Dialog.okButton()]
            });
        };
        // Display error dialog when inserting a code snippet into unsupported widget (i.e. not an editor)
        this.showErrDialog = (errMsg) => {
            return showDialog({
                title: 'Error',
                body: errMsg,
                buttons: [Dialog.okButton()]
            });
        };
        // Bold text in snippet name based on search
        this.boldNameOnSearch = (searchValue, language, name) => {
            const displayName = language + name;
            if (searchValue !== '' &&
                displayName.toLowerCase().includes(searchValue.toLowerCase())) {
                const startIndex = displayName
                    .toLowerCase()
                    .indexOf(searchValue.toLowerCase());
                const endIndex = startIndex + searchValue.length;
                if (endIndex <= language.length) {
                    return React.createElement("span", null, name);
                }
                else {
                    const start = displayName.substring(language.length, startIndex);
                    const bolded = displayName.substring(startIndex, endIndex);
                    const end = displayName.substring(endIndex);
                    return (React.createElement("span", null,
                        start,
                        React.createElement("mark", { className: SEARCH_BOLD }, bolded),
                        end));
                }
            }
            return React.createElement("span", null, name);
        };
        // Render display of code snippet list
        this.renderCodeSnippet = (codeSnippet, id) => {
            const buttonClasses = BUTTON_CLASS;
            const displayName = '[' + codeSnippet.language + '] ' + codeSnippet.name;
            const name = codeSnippet.name;
            const language = codeSnippet.language;
            const actionButtons = [
                {
                    title: 'Insert, copy, edit, and delete',
                    icon: moreOptionsIcon,
                    onClick: (event) => {
                        showMoreOptions({ body: new OptionsHandler(this, codeSnippet) });
                        this._setOptionsPosition(event);
                    }
                }
            ];
            return (React.createElement("div", { key: codeSnippet.name, className: CODE_SNIPPET_ITEM, id: id },
                React.createElement("div", { className: CODE_SNIPPET_METADATA },
                    React.createElement("div", { key: displayName, className: TITLE_CLASS, id: id },
                        React.createElement("div", { id: id, title: codeSnippet.name, className: DISPLAY_NAME_CLASS },
                            this.renderLanguageIcon(codeSnippet.language),
                            this.boldNameOnSearch(this.state.searchValue, language, name)),
                        React.createElement("div", { className: ACTION_BUTTONS_WRAPPER_CLASS, id: id }, actionButtons.map(btn => {
                            return (React.createElement("button", { key: btn.title, title: btn.title, className: buttonClasses + ' ' + ACTION_BUTTON_CLASS, onClick: (event) => {
                                    btn.onClick(event);
                                } },
                                React.createElement(btn.icon.react, { tag: "span", elementPosition: "center", width: "16px", height: "16px" })));
                        }))))));
        };
        // static getDerivedStateFromProps(
        //   props: ICodeSnippetDisplayProps,
        //   state: ICodeSnippetDisplayState
        // ): ICodeSnippetDisplayState {
        //   if (state.searchValue === '' && state.filterTags.length === 0) {
        //     return {
        //       codeSnippets: props.codeSnippets,
        //       searchValue: '',
        //       filterTags: [],
        //       hidden: 'is_hidden'
        //     };
        //   }
        //   if (state.searchValue !== '' || state.filterTags.length !== 0) {
        //     const newSnippets = props.codeSnippets.filter(codeSnippet => {
        //       return (
        //         (state.searchValue !== '' &&
        //           codeSnippet.name.toLowerCase().includes(state.searchValue)) ||
        //         (state.searchValue !== '' &&
        //           codeSnippet.language.toLowerCase().includes(state.searchValue)) ||
        //         (codeSnippet.tags &&
        //           codeSnippet.tags.some(tag => state.filterTags.includes(tag)))
        //       );
        //     });
        //     return {
        //       codeSnippets: newSnippets,
        //       searchValue: state.searchValue,
        //       filterTags: state.filterTags,
        //       hidden: 'is_hidden'
        //     };
        //   }
        //   return null;
        // }
        this.filterSnippets = (searchValue, filterTags) => {
            // filter with search
            let filteredSnippets = this.props.codeSnippets.filter(codeSnippet => codeSnippet.name.toLowerCase().includes(searchValue.toLowerCase()) ||
                codeSnippet.language.toLowerCase().includes(searchValue.toLowerCase()));
            // filter with tags
            if (filterTags.length !== 0) {
                filteredSnippets = filteredSnippets.filter(codeSnippet => {
                    return filterTags.some(tag => {
                        if (codeSnippet.tags) {
                            return codeSnippet.tags.includes(tag);
                        }
                        return false;
                    });
                });
            }
            this.setState({
                codeSnippets: filteredSnippets,
                searchValue: searchValue,
                filterTags: filterTags
            });
        };
        this.state = {
            codeSnippets: this.props.codeSnippets,
            searchValue: '',
            filterTags: [],
            hidden: 'is_hidden',
            snippetList: []
        };
        this._drag = null;
        this._dragData = null;
    }
    // 组件挂载
    componentDidMount() {
        this.getSnippetsList();
    }
    ;
    // 获取python函数列表
    getSnippetsList() {
        requestAPI(`functions`).then((res) => {
            console.log(res);
            if (res.code === 200) {
                const source = res.result.data.list;
                const list = source.map((item) => {
                    return {
                        id: item.id,
                        name: item.name,
                        language: 'python',
                        description: '',
                        code: [item.name],
                        tags: [item.name]
                    };
                });
                this.setState({ hidden: "" });
                this.setState({ snippetList: list });
            }
            else {
                alert('error');
            }
        }).catch(err => {
            alert(err.message);
        });
    }
    //Set the position of the option to be under to the three dots on snippet.
    _setOptionsPosition(event) {
        const target = event.target;
        let top;
        if (target.tagName === 'path') {
            top = target.getBoundingClientRect().top + 10;
        }
        else {
            top = target.getBoundingClientRect().top + 18;
        }
        if (top > 0.7 * window.screen.height) {
            top -= 120;
        }
        const leftAsString = target.getBoundingClientRect().left.toString(10) + 'px';
        const topAsString = top.toString(10) + 'px';
        document.documentElement.style.setProperty('--more-options-top', topAsString);
        document.documentElement.style.setProperty('--more-options-left', leftAsString);
    }
    renderLanguageIcon(language) {
        return React.createElement(pythonIcon.react, { tag: "span", height: "16px", width: "16px", right: "7px", top: "5px", "margin-right": "3px" });
    }
    getActiveTags() {
        const tags = [];
        for (const codeSnippet of this.props.codeSnippets) {
            if (codeSnippet.tags) {
                for (const tag of codeSnippet.tags) {
                    if (!tags.includes(tag)) {
                        tags.push(tag);
                    }
                }
            }
        }
        return tags;
    }
    // remove dropdown menu
    removeOptionsNode() {
        const temp = document.getElementsByClassName(CODE_SNIPPET_MORE_OPTIONS)[0];
        if (!temp.classList.contains('inactive')) {
            temp.classList.add('inactive');
        }
    }
    // create dropdown menu
    createOptionsNode(codeSnippet) {
        const body = document.createElement('div');
        const optionsContainer = document.createElement('div');
        optionsContainer.className = CODE_SNIPPET_MORE_OTPIONS_CONTENT;
        const insertSnip = document.createElement('div');
        insertSnip.className = CODE_SNIPPET_MORE_OTPIONS_INSERT;
        insertSnip.textContent = 'Insert dataset';
        insertSnip.onclick = () => {
            this.insertCodeSnippet(codeSnippet);
            this.removeOptionsNode();
        };
        const copySnip = document.createElement('div');
        copySnip.className = CODE_SNIPPET_MORE_OTPIONS_COPY;
        copySnip.textContent = 'Copy dataset to clipboard';
        copySnip.onclick = () => {
            Clipboard.copyToSystem(`obs_utils.read_dataset(${codeSnippet.code.join('\n')})`);
            alert('saved to clipboard');
            this.removeOptionsNode();
        };
        optionsContainer.appendChild(insertSnip);
        optionsContainer.appendChild(copySnip);
        body.append(optionsContainer);
        return body;
    }
    render() {
        return (React.createElement("div", null,
            React.createElement(FilterTools, { tags: this.getActiveTags(), onFilter: this.filterSnippets }),
            React.createElement("div", { className: this.state.hidden ? 'is_hidden_SnippetsContainer' : CODE_SNIPPETS_CONTAINER },
                React.createElement("div", null, this.state.snippetList.map((codeSnippet, id) => this.renderCodeSnippet(codeSnippet, id.toString()))))));
    }
}
class OptionsHandler extends Widget {
    constructor(display, codeSnippet) {
        super({ node: display.createOptionsNode(codeSnippet) });
    }
}
/**
 * The default implementation of an `IModelFactory`.
 */
export class ModelFactory {
    /**
     * Create a new code cell.
     *
     * @param source - The data to use for the original source data.
     *
     * @returns A new code cell. If a source cell is provided, the
     *   new cell will be initialized with the data from the source.
     *   If the contentFactory is not provided, the instance
     *   `codeCellContentFactory` will be used.
     */
    createCodeCell(options) {
        if (!options.contentFactory) {
            options.contentFactory = this.codeCellContentFactory;
        }
        return new CodeCellModel(options);
    }
}
